<?php
/**
 * sh404SEF - SEO extension for Joomla!
 *
 * @author      Yannick Gaultier
 * @copyright   (c) Yannick Gaultier - Weeblr llc - 2015
 * @package     sh404SEF
 * @license     http://www.gnu.org/copyleft/gpl.html GNU/GPL
 * @version     4.6.0.2718
 * @date        2015-07-14
 */

// Security check to ensure this file is being included by a parent file.
if (!defined('_JEXEC'))
	die('Direct Access to this location is not allowed.');

class Sh404sefHelperError
{

	/**
	 * Render an html document in response to a request
	 * that generated a 404 error
	 *
	 * @param Exception $error a 404 exception
	 * @param array $queryVars an array of query vars representing the page to display
	 * @param string $title an optional page title
	 */
	public static function render404ErrorDocument(& $error, $queryVars = array(), $title = '')
	{
		$queryVars = empty($queryVars) ? self::get404PageQueryVars() : $queryVars;

		if (!empty($queryVars))
		{
			$app = JFactory:: getApplication();
			foreach ($queryVars as $key => $value)
			{
				$app->input->set($key, $value);
			}

			// no caching on this page
			JFactory::getConfig()->set('caching', 0);

			// clean any previous output
			@ob_end_clean();
			ob_start();

			// pretend routing happened normally
			$app->triggerEvent('onAfterRoute');

			// make sure we have a 404 header
			if (!headers_sent())
			{
				JResponse::setHeader('status', '404 NOT FOUND', true);
			}

			// set Itemid if any requested, and article display params
			if (!empty($queryVars['Itemid']))
			{
				self::setItemid($queryVars['Itemid']);
			}

			// render page
			self::render($error, empty($title) ? JText::_('COM_SH404SEF_PAGE_NOT_FOUND_TITLE') : $title);
		}
		else
		{
			self::safeEcho($error->getMessage());
		}
	}

	/**
	 * Figure out the query vars that would make
	 * Joomla! display a given article
	 *
	 * @return array
	 */
	private static function get404PageQueryVars()
	{
		// get config objects
		$pageInfo = Sh404sefFactory::getPageInfo();
		$sefConfig = Sh404sefFactory::getConfig();

		// store the status
		$pageInfo->httpStatus = 404;

		// display the error page
		$vars['option'] = 'com_content';
		$vars['view'] = 'article';
		$vars['format'] = 'html';

		// use provided Itemid
		if (empty($sefConfig->shPageNotFoundItemid))
		{
			$homeItemid = JFactory::getApplication()->getMenu()->getDefault();
			$vars['Itemid'] = (empty($homeItemid)) ? null : $homeItemid->id;
		}
		else
		{
			$vars['Itemid'] = $sefConfig->shPageNotFoundItemid;
		}

		// user picked our default 404 error page, read its id from DB
		if ($sefConfig->page404 == '0')
		{
			try
			{
				$languageTag = JFactory::getLanguage()->getTag();
				if (empty($languageTag))
				{
					$languageTag = JRequest::getString(JApplication::getHash('language'), null, 'cookie');
				}

				if (!empty($languageTag))
				{
					$vars['lang'] = $languageTag;
				}
				$ids = ShlDbHelper::quoteQuery(
					'SELECT ?? FROM ?? WHERE ?? = ? AND ?? = ? AND ?? = ? AND ?? IN ( ?, ?) ORDER BY ?? DESC',
					array(
						'id',
						'#__content',
						'title',
						'catid',
						'state',
						'language',
						'language'
					),
					array(
						'__404__',
						Sh404sefHelperCategories::getSh404sefContentCat()->id,
						1,
						$languageTag,
						'*'
					))->loadColumn();
				$id = empty($ids[0]) ? null : $ids[0];
			}
			catch (Exception $e)
			{
				ShlSystem_Log::error('sh404sef', '%s::%s::%d: %s', __CLASS__, __METHOD__, __LINE__,
					' Database error: ' . $e->getMessage());
			}
			if (empty($id))
			{
				jexit('<strong>An error just happened: </strong>' . JText::_('Component Not Found') . ' (' . $pageInfo->getDefaultFrontLiveSite()
					. ')<br /><small>and there was another one displaying this message (error page document not found).</small>');
			}
		}
		else
		{
			$id = $sefConfig->page404;
		}
		$vars['id'] = $id;
		$vars['lang'] = empty($vars['lang']) ? null : Sh404sefHelperLanguage::getUrlCodeFromTag($vars['lang']);
		$tmpl = str_replace('.php', '', $sefConfig->error404SubTemplate);
		if (!empty($tmpl))
		{
			$vars['tmpl'] = $tmpl;
		}

		return $vars;
	}

	/**
	 * Set a given menu item as active
	 *
	 * @param int $Itemid
	 */
	public static function setItemid($Itemid)
	{
		$menus = JFactory::getApplication()->getMenu();
		$menuItem = $menus->getItem($Itemid);
		if (!empty($menuItem))
		{
			$menus->setActive($Itemid);
			// set options for proper display
			$params = array(
				'show_title',
				'show_vote',
				'show_tags',
				'show_publish_date',
				'show_print_icon',
				'show_email_icon',
				'show_parent_category',
				'show_page_heading',
				'show_noauth',
				'show_modify_date',
				'show_item_navigation',
				'show_icons',
				'show_hits',
				'show_create_date',
				'show_category',
				'show_author'
			);
			foreach ($params as $key)
			{
				$menuItem->params->set($key, 0);
			}
			$menuItem->params->set('show_intro', 1);

			// update global variables
			JFactory::getApplication()->getParams()->set('pageclass_sfx', $menuItem->params->get('pageclass_sfx'));
		}
	}

	/**
	 * Render a regular html document
	 *
	 * @param $error
	 * @param $title
	 */
	protected static function render($error, $title)
	{
		$app = JFactory:: getApplication();
		$document = JFactory::getDocument();

		// Get the template
		$template = $app->getTemplate(true);

		// Store the template and its params to the config
		$app->set('theme', $template->template);
		$app->set('themeParams', $template->params);

		// render component
		defined('JPATH_COMPONENT') or define('JPATH_COMPONENT', JPATH_ROOT . '/components/com_content');
		defined('JPATH_COMPONENT_SITE') or define('JPATH_COMPONENT_SITE', JPATH_ROOT . '/components/com_content');
		if(file_exists(JPATH_ROOT . '/components/com_content/helpers/route.php'))
		{
			require_once JPATH_ROOT . '/components/com_content/helpers/route.php';
		}
		if (file_exists(JPATH_ROOT . '/components/com_content/helpers/query.php'))
		{
			require_once JPATH_ROOT . '/components/com_content/helpers/query.php';
		}
		$controller = JControllerLegacy::getInstance('Content');
		$controller->execute($app->input->get('task'));
		$contents = ob_get_clean();
		$document->setBuffer($contents, 'component');
		$document->setTitle($title);

		// Trigger the onAfterDispatch event.
		JPluginHelper::importPlugin('system');
		$app->triggerEvent('onAfterDispatch');

		// Setup the document options.
		$docOptions['template'] = $template->template;
		$templateFile = $app->input->get('tmpl', 'index');
		$docOptions['file'] = $templateFile . '.php';
		$docOptions['params'] = $app->get('themeParams');

		if ($app->get('themes.base'))
		{
			$docOptions['directory'] = $app->get('themes.base');
		}
		// Fall back to constants.
		else
		{
			$docOptions['directory'] = defined('JPATH_THEMES') ? JPATH_THEMES : (defined('JPATH_BASE') ? JPATH_BASE : __DIR__) . '/themes';
		}

		// Fix base tag
		$document->setBase(htmlspecialchars(JUri::current()));

		// Parse the document.
		$document->parse($docOptions);

		// kill pathway
		$newPathWay = new stdClass();
		$newPathWay->name= JText::_('COM_SH404SEF_PAGE_NOT_FOUND_PATHWAY');
		$newPathWay->link = '';
		$app->getPathway()->setPathway(array($newPathWay));

		// Trigger the onBeforeRender event.
		JPluginHelper::importPlugin('system');
		$app->triggerEvent('onBeforeRender');

		$caching = false;

		// Render the document.
		$data = $document->render($caching, $docOptions);

		// Failsafe to get the error displayed.
		if (empty($data))
		{
			self::safeEcho($error->getMessage());
		}
		else
		{
			if(version_compare(JVERSION, '3', 'ge'))
			{
				$app->setBody($data);
			}
			else
			{
				JResponse::setBody($data);
			}

			// Trigger the onAfterRender event.
			$app->triggerEvent('onAfterRender');

			echo $app->toString();
		}
	}

	/**
	 * Displays a message to screen or whatever is available
	 * in a way that should work in most situations
	 *
	 * @param $message string to display
	 */
	public static function safeEcho($message)
	{
		if (isset($_SERVER['HTTP_HOST']))
		{
			// Output as html
			echo "<br /><b>Error:</b>: $message<br />\n";
		}
		else
		{
			// Output as simple text
			if (defined('STDERR'))
			{
				fwrite(STDERR, "$message\n");
			}
			else
			{
				echo "$message\n";
			}
		}
	}
}
